# Copyright 2013 David Reichard
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###############################################################################

from gamelib.scene_object import SceneObject
from gamelib.timer import Timer
import gamelib.data
from gamelib.vector import Vec2d
from game.player import Player
import random

SPONGE_IMAGE = gamelib.data.load_image("sponge.png", None, True)
SHARK_IMAGE = gamelib.data.load_image("shark.png", None, True)
SHARK_WIDTH_HALF = SHARK_IMAGE.get_width() / 2
NET_IMAGE = gamelib.data.load_image("net.png", None, True)

SPONGE_SOUND = gamelib.data.load_sound("chitter")
SHARK_SOUND = gamelib.data.load_sound("shark_impact", "ogg")
JELLY_SOUND = gamelib.data.load_sound("quick_zap")
NET_SOUND = gamelib.data.load_sound("net")

SHARK_ATTACK_DISTANCE = 100 ** 2
SHARK_FOV = 60


class Sponge(SceneObject):
    def __init__(self, pos):
        SceneObject.__init__(self, SPONGE_IMAGE, None, pos)

    def on_collide(self):
        SPONGE_SOUND.play()
        Player.increase_score(150 * self.parent.parent.difficulty)
        self.kill()


class Jellyfish(SceneObject):
    def __init__(self, pos):
        SceneObject.__init__(self, None, gamelib.data.load_animation_set("jellyfish"), pos)

    def on_add(self, parent):
        self.animation.set("swim")
        self.animation.start()
        self.velocity = Vec2d(-random.randint(50, 80), 0)
        parent.parent.listen("update", self.on_update)

    def on_collide(self):
        JELLY_SOUND.play()
        Player.increase_score(100 * self.parent.parent.difficulty)
        self.kill()

    def on_update(self, dt):
        self.translate(*(self.velocity * dt))


class Shark(SceneObject):
    def __init__(self, pos):
        SceneObject.__init__(self, SHARK_IMAGE, None, pos, 'center')
        self.direction = Vec2d(-1, 0)
        self.base_speed = random.randint(95, 130)
        self.speed = self.base_speed

    def on_add(self, parent):
        parent.parent.listen("update", self.on_update)

    def update_ai(self, player):
        # If the shark gets close to Penelope then lunge for her.
        pos = self.position - (SHARK_WIDTH_HALF, 0)
        distance = pos.get_dist_sqrd(player.position)
        if distance <= SHARK_ATTACK_DISTANCE:
            direction_to_player = (player.position - pos).normalized()
            angle = Vec2d(-1, 0).get_angle_between2(direction_to_player)
            if angle <= SHARK_FOV:
                self.direction = direction_to_player
                if self.speed < 200:
                    self.speed = 200
        else:
            if self.speed > self.base_speed:
                self.speed = self.base_speed
            if self.direction != (-1, 0):
                self.direction = Vec2d(-1, 0)

    def on_collide(self):
        Player.damage(1)
        SHARK_SOUND.play()
        self.kill()

    def on_update(self, dt):
        player = self.parent.parent.groups["player"].get_sprite(0)
        self.update_ai(player)
        self.translate(*(self.direction*self.speed*dt))


class Net(SceneObject):
    def __init__(self, pos):
        SceneObject.__init__(self, NET_IMAGE, None, pos)

    def on_collide(self):
        Player.damage(1)
        NET_SOUND.play()
        self.kill()